from h5py import File, Group
from thd_json import projection_geometry
from h5schemas.scan_schema import open_schema
from h5schemas.initialize_dataset import initialize_dataset
from h5schemas.header.add_header import add_header_sample


def init_projection_geometry(
    h5_file: File | Group,
):
    schema_groups = get_projection_geometry_schemas()
    initialize_dataset(h5_file, schema_groups)


def get_projection_geometry_schemas():
    return open_schema(projection_geometry, "projection_geometry.json")


def map_update(values):
    dst, value = values
    dst.resize((dst.shape[0] + 1,))
    dst[-1] = value


def add_projection_geometry_sample(
    h5_file: File | Group,
    detector_center_position_mm: tuple[float, float, float],
    detector_center_orientation_quat: tuple[float, float, float, float],
    focal_spot_position_mm: tuple[float, float, float],
    focal_spot_orientation_quat: tuple[float, float, float, float] | None = None,
    uuid: str | None = None,
    timestamp: float | None = None,
):
    update_values = list()

    update_values.append(
        [h5_file["detector_center_position_mm/x"], detector_center_position_mm[0]]
    )
    update_values.append(
        [h5_file["detector_center_position_mm/y"], detector_center_position_mm[1]]
    )
    update_values.append(
        [h5_file["detector_center_position_mm/z"], detector_center_position_mm[2]]
    )

    update_values.append(
        [
            h5_file["detector_center_orientation_quat/x"],
            detector_center_orientation_quat[0],
        ]
    )
    update_values.append(
        [
            h5_file["detector_center_orientation_quat/y"],
            detector_center_orientation_quat[1],
        ]
    )
    update_values.append(
        [
            h5_file["detector_center_orientation_quat/z"],
            detector_center_orientation_quat[2],
        ]
    )
    update_values.append(
        [
            h5_file["detector_center_orientation_quat/w"],
            detector_center_orientation_quat[3],
        ]
    )

    update_values.append(
        [h5_file["focal_spot_position_mm/x"], focal_spot_position_mm[0]]
    )
    update_values.append(
        [h5_file["focal_spot_position_mm/y"], focal_spot_position_mm[1]]
    )
    update_values.append(
        [h5_file["focal_spot_position_mm/z"], focal_spot_position_mm[2]]
    )

    if focal_spot_orientation_quat is None:
        focal_spot_orientation_quat = (0.0, 0.0, 0.0, 1.0)

    update_values.append(
        [h5_file["focal_spot_orientation_quat/x"], focal_spot_orientation_quat[0]]
    )
    update_values.append(
        [h5_file["focal_spot_orientation_quat/y"], focal_spot_orientation_quat[1]]
    )
    update_values.append(
        [h5_file["focal_spot_orientation_quat/z"], focal_spot_orientation_quat[2]]
    )
    update_values.append(
        [h5_file["focal_spot_orientation_quat/w"], focal_spot_orientation_quat[3]]
    )

    for value in update_values:
        map_update(value)

    add_header_sample(h5_file["header"], uuid, timestamp)


if __name__ == "__main__":
    test_file = File("./examples/projection_geometry.h5", "w")
    init_projection_geometry(test_file)
    add_projection_geometry_sample(
        test_file,
        [0.0, 1.0, 2.0],
        [3.0, 4.0, 5.0, 6.0],
        [7.0, 8.0, 9.0],
        [10.0, 11.0, 12.0, 13],
    )
    add_projection_geometry_sample(
        test_file, [0.0, 1.0, 2.0], [3.0, 4.0, 5.0, 6.0], [7.0, 8.0, 9.0]
    )
